#<profile4xi profile = "02" />
# -------------------------------------------------------------------------------------------------
#<program4xi program = "imapiterator4xi.py">
#<abstract4xi abstract = "provides class IMapIterator4Xi"/>
#<refer4xi reference = "pool.py"/>
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'IMapIterator4Xi' ] 
#"
#/>

#/>
# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
try:                 # system ones
    import os
    import sys
    import signal
except:
    pass
try:                 # threading ones       
    import threading                
    import _thread   
except:
    pass
try:
    import _winapi
except:
    pass

                
try:                 # most-common ones
    import queue                    
    import itertools
    import collections
    import time      
except:
    pass
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
try:                             # Binary ones
    import _multiprocessing
except:
    pass
try:                             # Python ones
    from multiprocessing.pool import IMapIterator
except:
    print("imapiterator4xi.py *** failed in from multiprocessing.pool import IMapIterator ***")
#"
#/>

# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from ace_object4xi import ace_object4xi

from timeouterror4xi import TimeoutError4Xi
import constant_family4xi

import constant_heap4xi
import constant_pool4xi
import constant_synchronize4xi
import constant_system4xi
from constant_pool4xi import next_job_counter_pool4xi

from iteratorproxy4xi import IteratorProxy4Xi

from reason_object4xi import reason_object4xi
from sense_object4xi import sense_object4xi
from timeout4xi import omega2timeout4xi

from print4xi import print4xi
from time4xi import time4xi    
#"
#/>

# =================================================================================================
#<class4xi class = "IMapIterator4Xi" base = "object" child = "None" synopsis = "sets object result" usage = "
#   pool.py
#       instances are returned by `Pool.imap()`                             
#       class IMapUnorderedIterator(IMapIterator): <--- inherited class in pool.py
#       function imap: result = IMapIterator(self._cache)
#
#   imapunorderediterator4xi.py:
#       class IMapUnorderedIterator4Xi(IMapIterator4Xi) <-- base class in inheritance
#
#   pool4xi.py:
#       result2xi = IMapIterator4Xi(self._cache)"
#>
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
# =================================================================================================
#
class IMapIterator4Xi(object):
    '''
    IMapIterator4Xi class: pool module function imap sets object result    
    '''
    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi exposure = "
    _exposed_ = ('next', 'sense', 'reason', 'ace')          
    #"
    #/>

    #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    IteratorProxy4Xi.__next__ = next  # method of this class is set to __next__ method of IteratorProxy4Xi
                                      # to be solved - see pool.py class, it has no class variables
    #"
    #/>
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-IMapIterator4Xi" usage = "initilaizes with self, cache">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # --------------------------------------------------------------------------------------------- 
    def __init__(self, cache):
        '''
        IMapIterator4Xi method __init__:  initilaizes with self, cache    
        '''
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/10)          
        # -----------------------------------------------------------------------------------------
        
        class2print = "IMapIterator4Xi"
        method2print = "__init__"
        
        message2print = "{(Step 1/10a) started" + "; init-of-IMapIterator4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
                        
        message2print = "{(Step 1/10b) passed" + "; cache-passed-to-init-of-IMapIterator4Xi: " + str(cache) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  

        message2print = "{(Step 1/10c) borning" + "; imapiterator2xi-of-IMapIterator4Xi-borning: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  

        message2print = "{(Step 1/10d) inheriting" + "; object-inheriting-by-imapiterator2xi-of-IMapIterator4Xi: " + str(object)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)         
                         
                                    
        message2print = "{(Step 1/10e) sharing" + "; IteratorProxy4Xi-next-sharing-by-imapiterator2xi-of-IMapIterator4Xi: " + str(IteratorProxy4Xi.__next__)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                      
                        
        # -----------------------------------------------------------------------------------------
        # (Step 2/10)           
        # -----------------------------------------------------------------------------------------
                                                        
        self._cond = threading.Condition(threading.Lock())
        
        message2print = "{(Step 2/10) set" + "; cond-set-by-init-of-IMapIterator4Xi to: " + str(self._cond)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
                
        
        # -----------------------------------------------------------------------------------------
        # (Step 3/10)           
        # -----------------------------------------------------------------------------------------
                             
        self._job = next_job_counter_pool4xi()                       #dragonized
        message2print = "{(Step 3/10) set" + "; job-set-by-init-of-IMapIterator4Xi to: " + str(self._job)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # -----------------------------------------------------------------------------------------
        # (Step 4/10)           
        # -----------------------------------------------------------------------------------------
                             
        self._cache = cache
        message2print = "{(Step 4/10) set" + "; cache-set-by-init-of-IMapIterator4Xi to: " + str(self._cache)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # -----------------------------------------------------------------------------------------
        # (Step 5/10)           
        # -----------------------------------------------------------------------------------------
                             
        self._items = collections.deque()
        message2print = "{(Step 5/10) set" + "; items-set-by-init-of-IMapIterator4Xi to: " + str(self._items)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # -----------------------------------------------------------------------------------------
        # (Step 6/10)           
        # -----------------------------------------------------------------------------------------
                             
        self._index = 0
        message2print = "{(Step 6/10) set" + "; index-set-by-init-of-IMapIterator4Xi to: " + str(self._index)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # -----------------------------------------------------------------------------------------
        # (Step 7/10)           
        # -----------------------------------------------------------------------------------------
                             
        self._length = None
        message2print = "{(Step 7/10) set" + "; length-set-by-init-of-IMapIterator4Xi to: " + str(self._length)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # -----------------------------------------------------------------------------------------
        # (Step 8/10)           
        # -----------------------------------------------------------------------------------------
                             
        self._unsorted = {}
        message2print = "{(Step 8/10) set" + "; unsorted-set-by-init-of-IMapIterator4Xi to: " + str(self._unsorted)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # -----------------------------------------------------------------------------------------
        # (Step 9/10)           
        # -----------------------------------------------------------------------------------------
                             
        cache[self._job] = self
        message2print = "{(Step 9/10) set" + "; cache-job-set-by-init-of-IMapIterator4Xi to: " + str(cache[self._job])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # -----------------------------------------------------------------------------------------
        # (Step 10/10)           
        # -----------------------------------------------------------------------------------------
                             
        message2print = "{(Step 10/10a) activated" + "; imapiterator2xi-of-IMapIterator4Xi-activated: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
                             
        message2print = "{(Step 10/10b) shared" + "; IteratorProxy4Xi-next-shared-by-imapiterator2xi-of-IMapIterator4Xi: " + str(IteratorProxy4Xi.__next__)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
                             
        message2print = "{(Step 10/10c) returning" + "; init-of-IMapIterator4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                        
                
                                
    #</methodize4xi>
                                   
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "iter-of-IMapIterator4Xi" usage = "iterates with self">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # --------------------------------------------------------------------------------------------- 
    def __iter__(self):
        '''
        IMapIterator4Xi method __iter__: iterates with self    
        '''
        class2print = "IMapIterator4Xi"
        method2print = "__iter__"
        
        message2print = "{started" + "; iter-of-IMapIterator4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        
        message2print = "{returning" + "; imapiterator2xi-returning-by-iter-of-IMapIterator4Xi: " + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                                 
        
        return self
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "next-of-IMapIterator4Xi" usage = "nexts with timeout">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # --------------------------------------------------------------------------------------------- 
    def next(self, timeout=None):
        '''
        IMapIterator4Xi method next: nexts with timeout    
        '''
        class2print = "IMapIterator4Xi"
        method2print = "next"
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/4)           
        # -----------------------------------------------------------------------------------------
                     
        message2print = "{(Step 1/4a) started" + "; next-of-IMapIterator4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        
        timeout = omega2timeout4xi(class2print, method2print) 

        message2print = "{(Step 1/4b) set" + "; timeout-set-by-next-of-IMapIterator4Xi to: " + str(timeout)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
        
        self._cond.acquire()
        
        message2print = "{(Step 1/4c) performed" + "; cond-acquire-performed-by-next-of-IMapIterator4Xi: " + str(self._cond.acquire) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/4)       
        # -----------------------------------------------------------------------------------------
                               
        try:
            try:
                item = self._items.popleft()
                message2print = "{(Step 2/4a) set" + "; item-set-by-next-of-IMapIterator4Xi to: " + str(item)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                                
                                
            except IndexError:
                if self._index == self._length:
                    message2print = "{(Step 2/4z1) hit" + "; StopIteration-hit-by-next-of-IMapIterator4Xi: " + " *** " + str(StopIteration)  + " *** " + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    
                message2print = "{(Step 2/4b) passing" + "; timeout-passing-to-cond-wait-by-next-of-IMapIterator4Xi: " + str(timeout) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                
                                    
                self._cond.wait(timeout)
                
                message2print = "{(Step 2/4c) performed" + "; cond-wait-performed-by-next-of-IMapIterator4Xi: " + str(self._cond.wait) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                                
                try:
                    item = self._items.popleft()
                    
                    message2print = "{(Step 2/4d) set" + "; item-set-by-next-of-IMapIterator4Xi to: " + str(item)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
                                    
                                    
                except IndexError:
                    if self._index == self._length:
                        message2print = "{(Step 2/4z2) hit" + "; StopIteration-hit-by-next-of-IMapIterator4Xi: " + " *** " + str(StopIteration)  + " *** " + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
        
                    message2print = "{(Step 2/4z3) hit" + "; TimeoutError4Xi-hit-by-next-of-IMapIterator4Xi: " + " *** " + str(TimeoutError4Xi)  + " *** " + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                                        
                                        
                    
        finally:
            self._cond.release()
            
            message2print = "{(Step 2/4e) performed" + "; cond-release-performed-by-next-of-IMapIterator4Xi: " + str(self._cond.release) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
        
        # -----------------------------------------------------------------------------------------
        # (Step 3/4)      
        # -----------------------------------------------------------------------------------------
                                   
        success, value = item
        
        message2print = "{(Step 3/4a) set" + "; success-set-by-next-of-IMapIterator4Xi to: " + str(success)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
                
        message2print = "{(Step 3/4b) set" + "; value-set-by-next-of-IMapIterator4Xi to: " + str(value)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
                          
        
        # -----------------------------------------------------------------------------------------
        # (Step 4/4)         
        # -----------------------------------------------------------------------------------------
               
        if success:
            message2print = "{(Step 4/4a) returning" + "; value-returning-by-next-of-IMapIterator4Xi: " + str(value) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)    
                                    
            return value
        
        message2print = "{(Step 4/4z) hit" + "; value-hit-by-next-of-IMapIterator4Xi: " + " *** " + str(value)  + " *** " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)

        
    #</methodize4xi>
    
    
   
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "set-of-IMapIterator4Xi" usage = " sets with self, i, obj ">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # --------------------------------------------------------------------------------------------- 
    def _set(self, i, obj):
        '''
        IMapIterator4Xi method _set: sets with self, i, obj     
        '''
        class2print = "IMapIterator4Xi"
        method2print = "_set"
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/4)           
        # -----------------------------------------------------------------------------------------
            
        
        message2print = "{(Step 1/4a) started" + "; set-of-IMapIterator4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
           
        message2print = "{(Step 1/4b) passed" + "; i-passed-to-set-of-IMapIterator4Xi: " + str(i) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        
        message2print = "{(Step 1/4c) passed" + "; object-passed-to-set-of-IMapIterator4Xi: " + str(obj) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/4)           
        # -----------------------------------------------------------------------------------------
                      
        self._cond.acquire()
        
        message2print = "{(Step 2/4) performed" + "; cond-acquire-performed-by-set-of-IMapIterator4Xi: " + str(self._cond.acquire) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        
        # -----------------------------------------------------------------------------------------
        # (Step 3/4)           
        # -----------------------------------------------------------------------------------------
                            
        try:
            if self._index == i:
            
                self._items.append(obj)
                
                message2print = "{(Step 3/4a) appended" + "; object-appended-by-set-of-IMapIterator4Xi: " + str(obj) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                
                self._index += 1
                
                while self._index in self._unsorted:
                    
                    obj = self._unsorted.pop(self._index)
                    
                    message2print = "{(Step 3/4b) got" + "; object-got-by-set-of-IMapIterator4Xi: " + str(obj)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                                    
                    
                    self._items.append(obj)
                    
                    message2print = "{(Step 3/4c) appended" + "; object-appended-by-set-of-IMapIterator4Xi: " + str(obj) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                       
                    
                    self._index += 1
                    
                self._cond.notify()
                message2print = "{(Step 3/4d) performed" + "; cond-notify-performed-by-set-of-IMapIterator4Xi: " + str(self._cond.notify) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                                
            else:
                self._unsorted[i] = obj
                message2print = "{(Step 3/4e) set" + "; unsorted-i-set-by-set-of-IMapIterator4Xi to: " + str(self._unsorted[i])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                                
                                
            if self._index == self._length:
                message2print = "{(Step 3/4f) deleting" + "; cache-job-deleting-by-set-of-IMapIterator4Xi: " + str(self._cache[self._job]) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                                
                del self._cache[self._job]
        finally:
            self._cond.release()
            message2print = "{(Step 3/4g) performed" + "; cond-release-performed-by-set-of-IMapIterator4Xi: " + str(self._cond.release) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
        
        # -----------------------------------------------------------------------------------------
        # (Step 4/4)           
        # -----------------------------------------------------------------------------------------
                                
        message2print = "{(Step 4/4) returning" + "; set-of-IMapIterator4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                        
                
                               
    #</methodize4xi>
                        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "set-length-of-IMapIterator4Xi" usage = "sets length with self, length">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # --------------------------------------------------------------------------------------------- 
    def _set_length(self, length):
        '''
        IMapIterator4Xi method _set_length: sets length with self, length     
        '''
        class2print = "IMapIterator4Xi"
        method2print = "_set_length"
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/4)           
        # -----------------------------------------------------------------------------------------
                  
        message2print = "{(Step 1/4a) started" + "; set-length-of-IMapIterator4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        
        message2print = "{(Step 1/4b) passed" + "; length-passed-to-set-length-of-IMapIterator4Xi: " + str(length) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)     
                
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/4)           
        # -----------------------------------------------------------------------------------------
                   
        self._cond.acquire()
        message2print = "{(Step 2/4) performed" + "; cond-acquire-performed-by-set-length-of-IMapIterator4Xi: " + str(self._cond.acquire) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
                        
        
        # -----------------------------------------------------------------------------------------
        # (Step 3/4)           
        # -----------------------------------------------------------------------------------------
           
        try:
            self._length = length
            message2print = "{(Step 3/4a) set" + "; length-set-by-set-length-of-IMapIterator4Xi to: " + str(self._length)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                
                            
            if self._index == self._length:
                self._cond.notify()
                
                message2print = "{(Step 3/4b) performed" + "; cond-notify-performed-by-set-length-of-IMapIterator4Xi: " + str(self._cond.notify) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                                
                message2print = "{(Step 3/4c) deleting" + "; cache-job--deleting-by-set-length-of-IMapIterator4Xi: " + str(self._cache[self._job]) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                                
                del self._cache[self._job]
        finally:
            self._cond.release()
            
            message2print = "{(Step 3/4d) performed" + "; cond.release-performed-by-set-length-of-IMapIterator4Xi: " + str(self._cond.release) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
        
        # -----------------------------------------------------------------------------------------
        # (Step 4/4)           
        # -----------------------------------------------------------------------------------------
                   
        message2print = "{(Step 4/4) returning" + "; set-length-of-IMapIterator4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                        
                
                        
    #</methodize4xi>
                                           
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-IMapIterator4Xi" usage = "senses IMapIterator4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        method sense: senses IMapIterator4Xi class objects
        '''

        
        class2print = 'IMapIterator4Xi'
        method2print = 'sense'
        
        
        message2print = "{started" + "; sense-of-IMapIterator4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
   
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------  
        message2print = "{sensing" + "; IMapIterator4Xi-sensing-by-sense-of-IMapIterator4Xi: " + str(IMapIterator4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                     
        
        # -----------------------------------------------------------------------------------------
        # get sensed knowledge for this object
        # -----------------------------------------------------------------------------------------        
        kb2xi = sense_object4xi(self)
        
        message2print = "{got" + "; kb2xi-got-by-sense-of-IMapIterator4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb2xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------         
        try:
            kb2xi['next-of-iteratorproxy4xi'] = IteratorProxy4Xi.__next__
        except:
            message2print = "{yielded" + "; kb2xi-yielded-by-sense-of-IMapIterator4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
        
      
        message2print = "{sensed" + "; IMapIterator4Xi-sensed-by-sense-of-IMapIterator4Xi: " + str(IMapIterator4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        
        # -----------------------------------------------------------------------------------------
        # 
        # -----------------------------------------------------------------------------------------         
        message2print = "{returning" + "; kb2xi-returning-by-sense-of-IMapIterator4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                        
        return kb2xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
    #</methodize4xi>
    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-IMapIterator4Xi" usage = "reasons IMapIterator4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        method reason: reasons IMapIterator4Xi class objects
        '''
         
        
        class2print = 'IMapIterator4Xi'
        method2print = 'reason'

        
        message2print = "{started" + "; reason-of-IMapIterator4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
      
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{reasoning" + "; IMapIterator4Xi-reasoning-by-reason-of-IMapIterator4Xi: " + str(IMapIterator4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
                
        
        # -----------------------------------------------------------------------------------------
        # get reasoned knowledge for this object
        # -----------------------------------------------------------------------------------------         
        kb4xi = reason_object4xi(self)
        
        message2print = "{got" + "; kb4xi-got-by-reason-of-IMapIterator4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb4xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------          
        
        
          
        
        message2print = "{reasoned" + "; IMapIterator4Xi-reasoned-by-reason-of-IMapIterator4Xi: " + str(IMapIterator4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{got" + "; kb4xi-got-by-reason-of-IMapIterator4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                           
          
        return kb4xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-IMapIterator4Xi" usage = "aces IMapIterator4Xi class objects ">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        method ace: aces IMapIterator4Xi class objects 
        '''
              
        
        
        class2print = 'IMapIterator4Xi'
        method2print = 'ace'
        
        
        message2print = "{started" + "; ace-of-IMapIterator4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
           
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{acing" + "; IMapIterator4Xi-acing-by-ace-of-IMapIterator4Xi: " + str(IMapIterator4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                      
        
        # -----------------------------------------------------------------------------------------
        # get aced knowledge for this object
        # -----------------------------------------------------------------------------------------        
        kb8xi = ace_object4xi(self)  
        
        message2print = "{got" + "; kb8xi-got-by-ace-of-IMapIterator4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                


        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb8xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------              
        
          
        message2print = "{aced" + "; IMapIterator4Xi-aced-by-ace-of-IMapIterator4Xi: " + str(IMapIterator4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb8xi-returning-by-ace-of-IMapIterator4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                                   
        
        return kb8xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
#</class4xi>
#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   test_none4xi.txt
#   sense-test-none4xi.xml
#   reason-test-none4xi.xml
#   ace-test-none4xi.xml
#"
#>


# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>
 
#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of imapiterator4xi.py                             
# ------------------------------------------------------------------------------------------------- 
#"
#/>

 